function [path,delta,psi,score] = viterbi_colFLT(M, obslik)
% VITERBI Find the most-probable (Viterbi) path through the HMM states (frequency bins).
% 
% Inputs:
% M      - One dimension of the colfilt matrix, in the current model M=3, 
%          but keep it as an input for generic purpose
% obslik - Observation likelihood, i.e. Fstat
%
% Outputs:
% path(n,t)  - The nth best path, where path(n,1), path(n,2) ... path(n,T) stands for 
%              the frequency bin index for each step in the nth best path.
% delta(j,t) - Max probability of the sequence of length t-1 and then going to state j
% psi(j,t)   - The best predecessor state, given that we ended up in state j at t
% score      - Number of standard deviations above the mean value of log likelihood 
%              of paths to all the states at the final step
            

if round(M/2)==M/2 M=M+1;end
[Q,T]=size(obslik); % Q bins, T steps

delta = zeros(Q,T);
psi = zeros(Q,T);
path = zeros(1,T);

% Using Matlab function colfilt, which rearranges each Mx1 block
% of delta into a column of a temporary matrix and then applies
% the max or argmax function to this matrix. This function actually
% compares all three possible paths for each step and the operation
% is efficient.
       
t=1;
delta(:,t) = obslik(:,t);
cor=(0:Q-1)'-(M-1)/2; % correction term for each bin

disp('forward')
for t=2:T
  delta(:,t) = colfilt(delta(:,t-1),[M 1],'sliding',@max)+obslik(:,t);
  psi(:,t) = colfilt(delta(:,t-1),[M 1],'sliding',@argmax)+cor;
end

% Sort the paths
disp('backward')

% Only identify the optimal path
[sc,path(T)]=max(delta(:,T));
score=(sc-mean(delta(:,T)))/std(delta(:,T));

for t=T-1:-1:1
    path(t) = psi(path(t+1),t+1);
end

